"""
Global pytest fixtures
"""

# Use this file if you need to share any fixtures
# across multiple modules
# More information at
# https://docs.pytest.org/en/stable/how-to/fixtures.html#scope-sharing-fixtures-across-classes-modules-packages-or-session

import pytest

from transport_analysis.data.files import MDANALYSIS_LOGO


@pytest.fixture
def mdanalysis_logo_text() -> str:
    """Example fixture demonstrating how data files can be accessed"""
    with open(MDANALYSIS_LOGO, "r", encoding="utf8") as f:
        logo_text = f.read()
    return logo_text



"""
Location of data files
======================

Use as ::

    from transport_analysis.data.files import *

"""

__all__ = [
    "MDANALYSIS_LOGO",  # example file of MDAnalysis logo
]

from pkg_resources import resource_filename

MDANALYSIS_LOGO = resource_filename(__name__, "mda.txt")



import pytest
from numpy.testing import assert_allclose

from transport_analysis.viscosity import (
    ViscosityHelfand as VH,
)
import MDAnalysis as mda
from MDAnalysis.units import constants
from MDAnalysis.transformations import set_dimensions
import numpy as np

from MDAnalysis.exceptions import NoDataError
from MDAnalysisTests.datafiles import PRM_NCBOX, TRJ_NCBOX, PSF, DCD


@pytest.fixture(scope="module")
def u():
    return mda.Universe(PRM_NCBOX, TRJ_NCBOX)


@pytest.fixture(scope="module")
def ag(u):
    return u.select_atoms("name O and resname WAT and resid 1-10")


@pytest.fixture(scope="module")
def u_no_vels():
    return mda.Universe(PSF, DCD)


@pytest.fixture(scope="module")
def ag_no_vels(u_no_vels):
    return u_no_vels.select_atoms("backbone and name CA and resid 1-10")


@pytest.fixture(scope="module")
def NSTEP():
    nstep = 5001
    return nstep


@pytest.fixture(scope="module")
def visc_helfand(ag):
    vh_t = VH(ag)
    vh_t.run()
    return vh_t


# Full step trajectory of unit velocities i.e. v = 0 at t = 0,
# v = 1 at t = 1, v = 2 at t = 2, etc. for all components x, y, z
# with mass, positions, and volume
@pytest.fixture(scope="module")
def step_vtraj_full(NSTEP):
    # Set up unit velocities
    v = np.arange(NSTEP)
    velocities = np.vstack([v, v, v]).T
    # NSTEP frames x 1 atom x 3 dimensions, where NSTEP = 5001
    velocities_reshape = velocities.reshape([NSTEP, 1, 3])

    # Positions derived from unit velocity setup
    x = np.arange(NSTEP).astype(np.float64)
    # Since initial position and velocity are 0 and acceleration is 1,
    # position = 1/2t^2
    x *= x / 2
    positions = np.vstack([x, x, x]).T
    # NSTEP frames x 1 atom x 3 dimensions, where NSTEP = 5001
    positions_reshape = positions.reshape([NSTEP, 1, 3])
    u = mda.Universe.empty(1, n_frames=NSTEP, velocities=True)

    # volume of 8.0
    dim = [2, 2, 2, 90, 90, 90]
    for i, ts in enumerate(u.trajectory):
        u.atoms.velocities = velocities_reshape[i]
        u.atoms.positions = positions_reshape[i]
        set_dimensions(dim)(u.trajectory.ts)

    # mass of 16.0
    u.add_TopologyAttr("masses", [16.0])
    return u


def characteristic_poly_helfand(
    test_universe,
    stop,
    n_dim,
    temp_avg=300.0,
    mass=16.0,
    vol_avg=8.0,
    start=0,
    step=1,
):
    # update when mda 2.6.0 releases with typo fix (MDAnalysis Issue #4213)
    try:
        boltzmann = constants["Boltzmann_constant"]
    except KeyError:
        boltzmann = constants["Boltzman_constant"]

    d = stop - start
    frames_used = d // step + 1 if d % step != 0 else d / step
    frames_used = int(frames_used)

    result = np.zeros((frames_used))
    keys = {
        1: [0],
        2: [0, 1],
        3: [0, 1, 2],
    }
    velocities = np.zeros((frames_used, 1, n_dim))
    positions = np.zeros((frames_used, 1, n_dim))

    for i, ts in enumerate(test_universe.trajectory[start:stop:step]):
        velocities[i] = ts.velocities[:, keys[n_dim]]
        positions[i] = ts.positions[:, keys[n_dim]]

    for lag in range(1, frames_used):
        diff = mass * (
            velocities[:-lag, :, :] * positions[:-lag, :, :]
            - velocities[lag:, :, :] * positions[lag:, :, :]
        )

        sq_diff = np.square(diff).sum(axis=-1)
        result[lag] = np.mean(sq_diff, axis=0)

    result = result / (2 * boltzmann * vol_avg * temp_avg)
    return result


class TestViscosityHelfand:
    def test_ag_accepted(self, ag):
        VH(ag)

    def test_no_velocities(self, ag_no_vels):
        errmsg = "Helfand viscosity computation requires"
        with pytest.raises(NoDataError, match=errmsg):
            v = VH(ag_no_vels)
            v.run()

    def test_updating_ag_rejected(self, u):
        updating_ag = u.select_atoms("around 3.5 resid 1", updating=True)
        errmsg = "UpdatingAtomGroups are not valid"
        with pytest.raises(TypeError, match=errmsg):
            VH(updating_ag)

    @pytest.mark.parametrize("dimtype", ["foo", "bar", "yx", "zyx"])
    def test_dimtype_error(self, ag, dimtype):
        errmsg = f"invalid dim_type: {dimtype}"
        with pytest.raises(ValueError, match=errmsg):
            VH(ag, dim_type=dimtype)


@pytest.mark.parametrize(
    "tdim, tdim_factor",
    [
        ("xyz", 3),
        ("xy", 2),
        ("xz", 2),
        ("yz", 2),
        ("x", 1),
        ("y", 1),
        ("z", 1),
    ],
)
class TestAllDims:
    def test_step_vtraj_all_dims(
        self, step_vtraj_full, NSTEP, tdim, tdim_factor
    ):
        # Helfand viscosity results should agree with the unit velocity traj
        # defined in characteristic_poly_helfand()
        vis_h = VH(step_vtraj_full.atoms, dim_type=tdim)
        vis_h.run()
        poly = characteristic_poly_helfand(step_vtraj_full, NSTEP, tdim_factor)
        assert_allclose(vis_h.results.timeseries, poly)

    def test_start_stop_step_all_dims(
        self,
        step_vtraj_full,
        tdim,
        tdim_factor,
        tstart=10,
        tstop=1000,
        tstep=10,
    ):
        # Helfand viscosity results should agree with the unit velocity traj
        # defined in characteristic_poly_helfand()
        # test start stop step is working correctly
        vis_h = VH(step_vtraj_full.atoms, dim_type=tdim)
        vis_h.run(start=tstart, stop=tstop, step=tstep)
        # polynomial must take offset start into account
        poly = characteristic_poly_helfand(
            step_vtraj_full, tstop, tdim_factor, start=tstart, step=tstep
        )
        assert_allclose(vis_h.results.timeseries, poly)



"""
Unit and regression test for the transport_analysis package.
"""

# Import package, test suite, and other packages as needed
import transport_analysis
import pytest
import sys


def test_transport_analysis_imported():
    """Sample test, will always pass so long as import statement worked"""
    assert "transport_analysis" in sys.modules


def test_mdanalysis_logo_length(mdanalysis_logo_text):
    """Example test using a fixture defined in conftest.py"""
    logo_lines = mdanalysis_logo_text.split("\n")
    assert len(logo_lines) == 46, "Logo file does not have 46 lines!"



import pytest
from numpy.testing import (
    assert_almost_equal,
    assert_allclose,
    assert_approx_equal,
)

from transport_analysis.velocityautocorr import (
    VelocityAutocorr as VACF,
)
import MDAnalysis as mda
import numpy as np
import tidynamics
from scipy import integrate
import MDAnalysis.analysis.msd as msd
from scipy.stats import linregress

from MDAnalysis.exceptions import NoDataError
from MDAnalysisTests.datafiles import PRM_NCBOX, TRJ_NCBOX


@pytest.fixture(scope="module")
def u():
    return mda.Universe(PRM_NCBOX, TRJ_NCBOX)


@pytest.fixture(scope="module")
def ag(u):
    return u.select_atoms("name O and resname WAT and resid 1-10")


@pytest.fixture(scope="module")
def NSTEP():
    nstep = 5001
    return nstep


@pytest.fixture(scope="module")
def vacf(ag):
    # non-fft VACF
    v = VACF(ag, fft=False)
    v.run()
    return v


# Step trajectory of unit velocities i.e. v = 0 at t = 0,
# v = 1 at t = 1, v = 2 at t = 2, etc. for all components x, y, z
@pytest.fixture(scope="module")
def step_vtraj(NSTEP):
    v = np.arange(NSTEP)
    velocities = np.vstack([v, v, v]).T
    # NSTEP frames x 1 atom x 3 dimensions, where NSTEP = 5001
    velocities_reshape = velocities.reshape([NSTEP, 1, 3])
    u = mda.Universe.empty(1, n_frames=NSTEP, velocities=True)
    for i, ts in enumerate(u.trajectory):
        u.atoms.velocities = velocities_reshape[i]
    return u


# Position trajectory corresponding to unit velocity trajectory
@pytest.fixture(scope="module")
def step_vtraj_pos(NSTEP):
    x = np.arange(NSTEP).astype(np.float64)
    # Since initial position and velocity are 0 and acceleration is 1,
    # position = 1/2t^2
    x *= x / 2
    positions = np.vstack([x, x, x]).T
    # NSTEP frames x 1 atom x 3 dimensions, where NSTEP = 5001
    positions_reshape = positions.reshape([NSTEP, 1, 3])
    u_pos = mda.Universe.empty(1)
    u_pos.load_new(positions_reshape)
    return u_pos


# Expected VACF results for step_vtraj
# At time t, VACF is:
# sum_{x=0}^{N - 1 - t} x*(x + t) * n_dim / n_frames
# n_dim = 3 (typically) and n_frames = total_frames - t
def characteristic_poly(last, n_dim, first=0, step=1):
    diff = last - first
    frames_used = diff // step + 1 if diff % step != 0 else diff / step
    frames_used = int(frames_used)
    result = np.zeros(frames_used)
    for t in range(first, last, step):
        sum = 0
        sum = np.dtype("float64").type(sum)
        lagtime = t - first
        for x in range(first, (last - lagtime), step):
            sum += x * (x + lagtime)
        current_index = int(lagtime / step)
        vacf = sum * n_dim / (frames_used - current_index)
        result[current_index] = vacf
    return result


@pytest.mark.parametrize(
    "tdim, tdim_keys", [(1, [0]), (2, [0, 1]), (3, [0, 1, 2])]
)
def test_characteristic_poly(step_vtraj, NSTEP, tdim, tdim_keys):
    # test `characteristic_poly()` against `tidynamics.acf()``

    # expected result from tidynamics.acf()
    # n_particles should be 1 unless modifying the test
    n_particles = len(step_vtraj.atoms)
    # 2D array of frames x particles
    expected = np.zeros((NSTEP, n_particles))
    # 3D array of frames x particles x dimensions
    step_velocities = np.zeros((NSTEP, n_particles, tdim))

    for i, ts in enumerate(step_vtraj.trajectory):
        step_velocities[i] = step_vtraj.atoms.velocities[:, tdim_keys]

    for n in range(n_particles):
        expected[:, n] = tidynamics.acf(step_velocities[:, n, :])

    # average over n_particles
    expected = expected.mean(axis=1)

    # result from characteristic_poly()
    actual = characteristic_poly(NSTEP, tdim)

    # compare actual and expected
    assert_almost_equal(actual, expected, decimal=4)


class TestVelocityAutocorr:
    # fixtures are helpful functions that set up a test
    # See more at https://docs.pytest.org/en/stable/how-to/fixtures.html
    def test_ag_accepted(self, ag):
        VACF(ag, fft=False)

    def test_no_velocities(self):
        u_no_vels = mda.Universe.empty(10, n_frames=5, velocities=False)
        errmsg = "VACF computation requires velocities"
        with pytest.raises(NoDataError, match=errmsg):
            v = VACF(u_no_vels.atoms, fft=False)
            v.run()

    def test_updating_ag_rejected(self, u):
        updating_ag = u.select_atoms("around 3.5 resid 1", updating=True)
        errmsg = "UpdatingAtomGroups are not valid"
        with pytest.raises(TypeError, match=errmsg):
            VACF(updating_ag, fft=False)

    @pytest.mark.parametrize("dimtype", ["foo", "bar", "yx", "zyx"])
    def test_dimtype_error(self, ag, dimtype):
        errmsg = f"invalid dim_type: {dimtype}"
        with pytest.raises(ValueError, match=errmsg):
            VACF(ag, dim_type=dimtype)

    def test_plot_vacf(self, vacf):
        # Expected data to be plotted
        x_exp = vacf.times
        y_exp = vacf.results.timeseries

        # Actual data returned from plot
        (line,) = vacf.plot_vacf()
        x_act, y_act = line.get_xydata().T

        assert_allclose(x_act, x_exp)
        assert_allclose(y_act, y_exp)

    def test_plot_vacf_labels(self, vacf):
        # Expected labels
        x_exp = "Time (ps)"
        y_exp = "Velocity Autocorrelation Function (Å^2 / ps^2)"

        # Actual labels returned from plot
        (line,) = vacf.plot_vacf()
        x_act = line.axes.get_xlabel()
        y_act = line.axes.get_ylabel()

        assert x_act == x_exp
        assert y_act == y_exp

    def test_plot_vacf_custom_labels(self, vacf):
        # Expected labels
        x_exp = "Custom x-label"
        y_exp = "Custom VACF"

        # Actual labels returned from plot
        (line,) = vacf.plot_vacf(xlabel=x_exp, ylabel=y_exp)
        x_act = line.axes.get_xlabel()
        y_act = line.axes.get_ylabel()

        assert x_act == x_exp
        assert y_act == y_exp

    def test_plot_vacf_start_stop_step(self, vacf, start=1, stop=9, step=2):
        # Expected data to be plotted
        x_exp = vacf.times[start:stop:step]
        y_exp = vacf.results.timeseries[start:stop:step]

        # Actual data returned from plot
        (line,) = vacf.plot_vacf(start=start, stop=stop, step=step)
        x_act, y_act = line.get_xydata().T

        assert_allclose(x_act, x_exp)
        assert_allclose(y_act, y_exp)

    def test_plot_vacf_exception(self, step_vtraj):
        v = VACF(step_vtraj.atoms, fft=False)
        errmsg = "Analysis must be run"
        with pytest.raises(RuntimeError, match=errmsg):
            v.plot_vacf()

    def test_self_diffusivity_gk_exception(self, step_vtraj):
        v = VACF(step_vtraj.atoms, fft=False)
        errmsg = "Analysis must be run"
        with pytest.raises(RuntimeError, match=errmsg):
            v.self_diffusivity_gk()

    def test_self_diffusivity_gk_odd_exception(self, step_vtraj):
        v = VACF(step_vtraj.atoms, fft=False)
        errmsg = "Analysis must be run"
        with pytest.raises(RuntimeError, match=errmsg):
            v.self_diffusivity_gk_odd()

    def test_plot_running_integral(self, vacf):
        # Expected data to be plotted
        x_exp = vacf.times
        y_exp = np.zeros(vacf.n_frames)

        for i in range(1, vacf.n_frames):
            y_exp[i] = (
                integrate.trapezoid(
                    vacf.results.timeseries[: i + 1], vacf.times[: i + 1]
                )
                / vacf.dim_fac
            )

        # Actual data returned from plot
        (line,) = vacf.plot_running_integral()
        x_act, y_act = line.get_xydata().T

        assert_allclose(x_act, x_exp)
        assert_allclose(y_act, y_exp)

    def test_plot_running_integral_labels(self, vacf):
        # Expected labels
        x_exp = "Time (ps)"
        y_exp = "Running Integral of the VACF (Å^2 / ps)"

        # Actual labels returned from plot
        (line,) = vacf.plot_running_integral()
        x_act = line.axes.get_xlabel()
        y_act = line.axes.get_ylabel()

        assert x_act == x_exp
        assert y_act == y_exp

    def test_plot_running_integral_custom_labels(self, vacf):
        # Expected labels
        x_exp = "Custom x-label"
        y_exp = "Custom Running Integral"

        # Actual labels returned from plot
        (line,) = vacf.plot_running_integral(xlabel=x_exp, ylabel=y_exp)
        x_act = line.axes.get_xlabel()
        y_act = line.axes.get_ylabel()

        assert x_act == x_exp
        assert y_act == y_exp

    def test_plot_running_integral_start_stop_step(
        self, vacf, start=1, stop=9, step=2
    ):
        t_range = range(start, stop, step)
        # Expected data to be plotted
        x_exp = vacf.times[start:stop:step]
        y_exp = np.zeros(len(t_range))

        for i, j in enumerate(t_range):
            if i > 0:
                y_exp[i] = (
                    integrate.trapezoid(
                        vacf.results.timeseries[start : j + 1 : step],
                        vacf.times[start : j + 1 : step],
                    )
                    / vacf.dim_fac
                )

        # Actual data returned from plot
        (line,) = vacf.plot_running_integral(start=start, stop=stop, step=step)
        x_act, y_act = line.get_xydata().T

        assert_allclose(x_act, x_exp)
        assert_allclose(y_act, y_exp)

    def test_plot_running_integral_exception(self, step_vtraj):
        v = VACF(step_vtraj.atoms, fft=False)
        errmsg = "Analysis must be run"
        with pytest.raises(RuntimeError, match=errmsg):
            v.plot_running_integral()


class TestVACFFFT(object):
    @pytest.fixture(scope="class")
    def vacf_fft(self, ag):
        # fft VACF
        v = VACF(ag, fft=True)
        v.run()
        return v

    def test_fft_vs_simple_default(self, vacf, vacf_fft):
        # testing on the PRM_NCBOX, TRJ_NCBOX trajectory
        timeseries_simple = vacf.results.timeseries
        timeseries_fft = vacf_fft.results.timeseries
        assert_almost_equal(timeseries_simple, timeseries_fft, decimal=4)

    def test_fft_vs_simple_default_per_particle(self, vacf, vacf_fft):
        # check fft and simple give same result per particle
        per_particle_simple = vacf.results.vacf_by_particle
        per_particle_fft = vacf_fft.results.vacf_by_particle
        assert_almost_equal(per_particle_simple, per_particle_fft, decimal=4)


@pytest.mark.parametrize(
    "tdim, tdim_factor",
    [
        ("xyz", 3),
        ("xy", 2),
        ("xz", 2),
        ("yz", 2),
        ("x", 1),
        ("y", 1),
        ("z", 1),
    ],
)
class TestAllDims:
    def test_simple_step_vtraj_all_dims(
        self, step_vtraj, NSTEP, tdim, tdim_factor
    ):
        # testing the "simple" windowed algorithm on unit velocity trajectory
        # VACF results should fit the polynomial defined in
        # characteristic_poly()
        v_simple = VACF(step_vtraj.atoms, dim_type=tdim, fft=False)
        v_simple.run()
        poly = characteristic_poly(NSTEP, tdim_factor)
        assert_almost_equal(v_simple.results.timeseries, poly, decimal=4)

    def test_simple_start_stop_step_all_dims(
        self,
        step_vtraj,
        tdim,
        tdim_factor,
        tstart=10,
        tstop=1000,
        tstep=10,
    ):
        # testing the simple "windowed" algorithm on unit velocity trajectory
        # defined in step_vtraj()
        # test start stop step is working correctly
        v_simple = VACF(step_vtraj.atoms, dim_type=tdim, fft=False)
        v_simple.run(start=tstart, stop=tstop, step=tstep)
        poly = characteristic_poly(
            tstop, tdim_factor, first=tstart, step=tstep
        )
        # polynomial must take offset start into account
        assert_almost_equal(v_simple.results.timeseries, poly, decimal=4)

    def test_self_diffusivity_step_vtraj_all_dims(
        self, step_vtraj, NSTEP, tdim, tdim_factor
    ):
        # testing self-diffusivity calculated from the VACF of the
        # "simple" windowed algorithm on unit velocity trajectory
        # Integration results should match a separate integration method
        # Simpson is used for the check
        v_simple = VACF(step_vtraj.atoms, dim_type=tdim, fft=False)
        v_simple.run()
        sd_actual = v_simple.self_diffusivity_gk()
        sd_expected = (
            integrate.simpson(
                characteristic_poly(NSTEP, tdim_factor), range(NSTEP)
            )
            / tdim_factor
        )
        # 24307638750.0 (act) agrees with 24307638888.888885 (exp) to 8 sig figs
        assert_approx_equal(sd_actual, sd_expected, significant=8)

    def test_self_diffusivity_start_stop_step_all_dims(
        self,
        step_vtraj,
        NSTEP,
        tdim,
        tdim_factor,
        tstart=10,
        tstop=1000,
        tstep=10,
    ):
        # testing self-diffusivity calculated from the VACF of the
        # "simple" windowed algorithm on unit velocity trajectory
        # check that start, stop, step is working correctly
        v_simple = VACF(step_vtraj.atoms, dim_type=tdim, fft=False)
        v_simple.run()
        sd_actual = v_simple.self_diffusivity_gk(
            start=tstart, stop=tstop, step=tstep
        )
        sd_expected = (
            integrate.simpson(
                characteristic_poly(NSTEP, tdim_factor)[tstart:tstop:tstep],
                range(NSTEP)[tstart:tstop:tstep],
            )
            / tdim_factor
        )
        # 7705160166.66 (act) agrees with 7705162888.88 (exp) to 6 sig figs
        assert_approx_equal(sd_actual, sd_expected, significant=6)

    def test_self_diffusivity_odd_step_vtraj_all_dims(
        self, step_vtraj, NSTEP, tdim, tdim_factor
    ):
        # testing self-diffusivity (simpson) calculated from the VACF of the
        # "simple" windowed algorithm on unit velocity trajectory
        v_simple = VACF(step_vtraj.atoms, dim_type=tdim, fft=False)
        v_simple.run()
        sd_actual = v_simple.self_diffusivity_gk_odd()
        sd_expected = (
            integrate.trapezoid(
                characteristic_poly(NSTEP, tdim_factor), range(NSTEP)
            )
            / tdim_factor
        )
        # 24307638750.0 (exp) agrees with 24307638888.888885 (act) to 8 sig figs
        assert_approx_equal(sd_actual, sd_expected, significant=8)

    def test_self_diffusivity_odd_start_stop_step_all_dims(
        self,
        step_vtraj,
        NSTEP,
        tdim,
        tdim_factor,
        tstart=10,
        tstop=1000,
        tstep=10,
    ):
        # testing self-diffusivity (simpson) calculated from the VACF of the
        # "simple" windowed algorithm on unit velocity trajectory
        # check that start, stop, step is working correctly
        v_simple = VACF(step_vtraj.atoms, dim_type=tdim, fft=False)
        v_simple.run()
        sd_actual = v_simple.self_diffusivity_gk_odd(
            start=tstart, stop=tstop, step=tstep
        )
        sd_expected = (
            integrate.trapezoid(
                characteristic_poly(NSTEP, tdim_factor)[tstart:tstop:tstep],
                range(NSTEP)[tstart:tstop:tstep],
            )
            / tdim_factor
        )
        # 7705160166.66 (exp) agrees with 7705162888.88 (act) to 6 sig figs
        assert_approx_equal(sd_actual, sd_expected, significant=6)

    def test_fft_step_vtraj_all_dims(
        self, step_vtraj, NSTEP, tdim, tdim_factor
    ):
        # testing the fft algorithm on unit velocity trajectory
        # defined in step_vtraj()
        # VACF results should fit the characteristic polynomial defined in
        # characteristic_poly()

        # fft based tests require a slight decrease in expected prescision
        # primarily due to roundoff in fft(ifft()) calls.
        # relative accuracy expected to be around ~1e-12
        v_fft = VACF(step_vtraj.atoms, dim_type=tdim, fft=True)
        v_fft.run()
        poly = characteristic_poly(NSTEP, tdim_factor)
        # this was relaxed from decimal=4 for numpy=1.13 test
        assert_almost_equal(v_fft.results.timeseries, poly, decimal=3)

    def test_fft_start_stop_step_all_dims(
        self, step_vtraj, tdim, tdim_factor, tstart=10, tstop=1000, tstep=10
    ):
        # testing the fft algorithm on unit velocity trajectory
        # defined in step_vtraj()
        # test start stop step is working correctly
        v_fft = VACF(step_vtraj.atoms, dim_type=tdim, fft=True)
        v_fft.run(start=tstart, stop=tstop, step=tstep)
        poly = characteristic_poly(
            tstop, tdim_factor, first=tstart, step=tstep
        )
        # polynomial must take offset start into account
        assert_almost_equal(v_fft.results.timeseries, poly, decimal=3)

    def test_fft_self_diffusivity_step_vtraj_all_dims(
        self, step_vtraj, NSTEP, tdim, tdim_factor
    ):
        # testing self-diffusivity calculated from the fft VACF of the
        # unit velocity trajectory
        # Integration results should match a separate integration method
        # Simpson is used for the check
        v_fft = VACF(step_vtraj.atoms, dim_type=tdim, fft=True)
        v_fft.run()
        sd_actual = v_fft.self_diffusivity_gk()
        sd_expected = (
            integrate.simpson(
                characteristic_poly(NSTEP, tdim_factor), range(NSTEP)
            )
            / tdim_factor
        )
        # 24307638750.0 (act) agrees with 24307638888.888885 (exp) to 8 sig figs
        assert_approx_equal(sd_actual, sd_expected, significant=8)

    def test_fft_self_diffusivity_start_stop_step_all_dims(
        self,
        step_vtraj,
        NSTEP,
        tdim,
        tdim_factor,
        tstart=10,
        tstop=1000,
        tstep=10,
    ):
        # testing self-diffusivity calculated from the VACF of the
        # "simple" windowed algorithm on unit velocity trajectory
        # check that start, stop, step is working correctly
        v_fft = VACF(step_vtraj.atoms, dim_type=tdim, fft=True)
        v_fft.run()
        sd_actual = v_fft.self_diffusivity_gk(
            start=tstart, stop=tstop, step=tstep
        )
        sd_expected = (
            integrate.simpson(
                characteristic_poly(NSTEP, tdim_factor)[tstart:tstop:tstep],
                range(NSTEP)[tstart:tstop:tstep],
            )
            / tdim_factor
        )
        # 7705160166.66 (act) agrees with 7705162888.88 (exp) to 6 sig figs
        assert_approx_equal(sd_actual, sd_expected, significant=6)

    def test_fft_self_diffusivity_odd_step_vtraj_all_dims(
        self, step_vtraj, NSTEP, tdim, tdim_factor
    ):
        # testing self-diffusivity (simpson) calculated from the fft VACF of the
        # unit velocity trajectory
        v_fft = VACF(step_vtraj.atoms, dim_type=tdim, fft=True)
        v_fft.run()
        sd_actual = v_fft.self_diffusivity_gk_odd()
        sd_expected = (
            integrate.trapezoid(
                characteristic_poly(NSTEP, tdim_factor), range(NSTEP)
            )
            / tdim_factor
        )
        # 24307638750.0 (exp) agrees with 24307638888.888885 (act) to 8 sig figs
        assert_approx_equal(sd_actual, sd_expected, significant=8)

    def test_fft_self_diffusivity_odd_start_stop_step_all_dims(
        self,
        step_vtraj,
        NSTEP,
        tdim,
        tdim_factor,
        tstart=10,
        tstop=1000,
        tstep=10,
    ):
        # testing self-diffusivity (simpson) calculated from the VACF of the
        # "simple" windowed algorithm on unit velocity trajectory
        # check that start, stop, step is working correctly
        v_fft = VACF(step_vtraj.atoms, dim_type=tdim, fft=True)
        v_fft.run()
        sd_actual = v_fft.self_diffusivity_gk_odd(
            start=tstart, stop=tstop, step=tstep
        )
        sd_expected = (
            integrate.trapezoid(
                characteristic_poly(NSTEP, tdim_factor)[tstart:tstop:tstep],
                range(NSTEP)[tstart:tstop:tstep],
            )
            / tdim_factor
        )
        # 7705160166.66 (exp) agrees with 7705162888.88 (act) to 6 sig figs
        assert_approx_equal(sd_actual, sd_expected, significant=6)

    def test_self_diffusivity_msd_all_dims(
        self, step_vtraj, step_vtraj_pos, tdim, tdim_factor
    ):
        # testing self-diffusivity calculated from the VACF (Green-Kubo)
        # against self-diffusivity calculated from the MSD (Einstein)

        # Green-Kubo self-diffusivity (actual)
        v_fft = VACF(step_vtraj.atoms, dim_type=tdim, fft=True)
        v_fft.run()
        sd_actual = v_fft.self_diffusivity_gk()

        # Einstein self-diffusivity (expected)
        MSD = msd.EinsteinMSD(step_vtraj_pos, select="all", msd_type=tdim)
        MSD.run()
        msd_res = MSD.results.timeseries
        lagtimes = np.arange(MSD.n_frames)
        start_time, end_time = 3000, 5000
        linear_model = linregress(
            lagtimes[start_time:end_time], msd_res[start_time:end_time]
        )
        sd_expected = linear_model.slope / (2 * tdim_factor)

        # 24307638750.0 (act) agrees with 24146066174.916477 (exp) to 2 sig figs
        assert_approx_equal(sd_actual, sd_expected, significant=2)



"""
Viscosity --- :mod:`transport_analysis.analysis.viscosity`
==========================================================

This module offers a class for the lightweight computation of shear
viscosity via the Einstein-Helfand method. It outputs the "viscosity
function," the product of viscosity and time as a function of time, from
which the slope is taken to calculate the shear viscosity. This is
described in eq. 5 of E M Kirova and G E Norman 2015 J. Phys.: Conf.
Ser. 653 012106.

"""
from typing import TYPE_CHECKING

from MDAnalysis.analysis.base import AnalysisBase
from MDAnalysis.core.groups import UpdatingAtomGroup
from MDAnalysis.exceptions import NoDataError
from MDAnalysis.units import constants
import numpy as np

if TYPE_CHECKING:
    from MDAnalysis.core.universe import AtomGroup


class ViscosityHelfand(AnalysisBase):
    """
    Class to calculate viscosity using the Einstein-Helfand approach.
    Note that the slope of the viscosity function, the product of viscosity
    and time as a function of time, must be taken to obtain the viscosity.

    Parameters
    ----------
    atomgroup : AtomGroup
        An MDAnalysis :class:`~MDAnalysis.core.groups.AtomGroup`.
        Note that :class:`UpdatingAtomGroup` instances are not accepted.
    temp_avg : float (optional, default 300)
        Average temperature over the course of the simulation, in Kelvin.
    dim_type : {'xyz', 'xy', 'yz', 'xz', 'x', 'y', 'z'}
        Desired dimensions to be included in the viscosity calculation.
        Defaults to 'xyz'.

    Attributes
    ----------
    atomgroup : :class:`~MDAnalysis.core.groups.AtomGroup`
        The atoms to which this analysis is applied
    dim_fac : int
        Dimensionality :math:`d` of the viscosity computation.
    results.timeseries : :class:`numpy.ndarray`
        The averaged viscosity function over all the particles with respect
        to lag-time. Obtained after calling :meth:`ViscosityHelfand.run`
    results.visc_by_particle : :class:`numpy.ndarray`
        The viscosity function of each individual particle with respect to
        lag-time.
    start : Optional[int]
        The first frame of the trajectory used to compute the analysis.
    stop : Optional[int]
        The frame to stop at for the analysis.
    step : Optional[int]
        Number of frames to skip between each analyzed frame.
    n_frames : int
        Number of frames analysed in the trajectory.
    n_particles : int
        Number of particles viscosity was calculated over.
    """

    def __init__(
        self,
        atomgroup: "AtomGroup",
        temp_avg: float = 300.0,
        dim_type: str = "xyz",
        **kwargs,
    ) -> None:
        # the below line must be kept to initialize the AnalysisBase class!
        super().__init__(atomgroup.universe.trajectory, **kwargs)
        # after this you will be able to access `self.results`
        # `self.results` is a dictionary-like object
        # that can should used to store and retrieve results
        # See more at the MDAnalysis documentation:
        # https://docs.mdanalysis.org/stable/documentation_pages/analysis/base.html?highlight=results#MDAnalysis.analysis.base.Results

        if isinstance(atomgroup, UpdatingAtomGroup):
            raise TypeError(
                "UpdatingAtomGroups are not valid for viscosity computation"
            )

        # args
        self.temp_avg = temp_avg
        self.dim_type = dim_type.lower()
        self._dim, self.dim_fac = self._parse_dim_type(self.dim_type)

        # local
        self.atomgroup = atomgroup
        self.n_particles = len(self.atomgroup)

    def _prepare(self):
        """
        Set up viscosity, mass, velocity, and position arrays
        before the analysis loop begins
        """
        # 2D viscosity array of frames x particles
        self.results.visc_by_particle = np.zeros(
            (self.n_frames, self.n_particles)
        )

        self._volumes = np.zeros((self.n_frames))

        self._masses = self.atomgroup.masses
        self._masses_rs = self._masses.reshape((1, len(self._masses), 1))

        # 3D arrays of frames x particles x dimensionality
        # for velocities and positions
        self._velocities = np.zeros(
            (self.n_frames, self.n_particles, self.dim_fac)
        )

        self._positions = np.zeros(
            (self.n_frames, self.n_particles, self.dim_fac)
        )
        # self.results.timeseries not set here

        # update when mda 2.6.0 releases with typo fix
        # (MDAnalysis Issue #4213)
        try:
            self.boltzmann = constants["Boltzmann_constant"]
        except KeyError:
            self.boltzmann = constants["Boltzman_constant"]

    @staticmethod
    def _parse_dim_type(dim_str):
        r"""Sets up the desired dimensionality of the calculation."""
        keys = {
            "x": [0],
            "y": [1],
            "z": [2],
            "xy": [0, 1],
            "xz": [0, 2],
            "yz": [1, 2],
            "xyz": [0, 1, 2],
        }

        try:
            _dim = keys[dim_str]
        except KeyError:
            raise ValueError(
                "invalid dim_type: {} specified, please specify one of xyz, "
                "xy, xz, yz, x, y, z".format(dim_str)
            )

        return _dim, len(_dim)

    def _single_frame(self):
        """
        Constructs arrays of velocities and positions
        for viscosity computation.
        """
        # This runs once for each frame of the trajectory

        # The trajectory positions update automatically
        # You can access the frame number using self._frame_index

        # trajectory must have velocity and position information
        if not (
            self._ts.has_velocities
            and self._ts.has_positions
            and self._ts.volume != 0
        ):
            raise NoDataError(
                "Helfand viscosity computation requires "
                "velocities, positions, and box volume in the trajectory"
            )

        # fill volume array
        self._volumes[self._frame_index] = self._ts.volume

        # set shape of velocity array
        self._velocities[self._frame_index] = self.atomgroup.velocities[
            :, self._dim
        ]

        # set shape of position array
        self._positions[self._frame_index] = self.atomgroup.positions[
            :, self._dim
        ]

    def _conclude(self):
        """
        Calculates the viscosity function via the simple "windowed" algorithm.
        """
        self._vol_avg = np.average(self._volumes)

        lagtimes = np.arange(1, self.n_frames)

        # iterate through all possible lagtimes from 1 to number of frames
        for lag in lagtimes:
            # get difference of momentum * position shifted by "lag" frames
            diff = (
                self._masses_rs
                * self._velocities[:-lag, :, :]
                * self._positions[:-lag, :, :]
                - self._masses_rs
                * self._velocities[lag:, :, :]
                * self._positions[lag:, :, :]
            )

            # square and sum each x(, y, z) diff per particle
            sq_diff = np.square(diff).sum(axis=-1)

            # average over # frames
            # update viscosity by particle array
            self.results.visc_by_particle[lag, :] = np.mean(sq_diff, axis=0)

        # divide by 2, boltzmann constant, vol_avg, and temp_avg
        self.results.visc_by_particle = self.results.visc_by_particle / (
            2 * self.boltzmann * self._vol_avg * self.temp_avg
        )
        # average over # particles and update results array
        self.results.timeseries = self.results.visc_by_particle.mean(axis=1)



"""
Velocity Autocorrelation Function --- :mod:`transport_analysis.velocityautocorr`
================================================================================

This module offers a class to efficiently calculate a velocity
autocorrelation function (VACF). Averaging over all atoms in the atom group
``ag``, regardless of type, it will calculate

.. math::
    C(j \Delta t) = {1 \over N - j} \sum_{i=0}^{N-1-j}
    v(i \Delta t)v((i+j)\Delta t)

where :math:`N` is the number of time frames and :math:`\Delta t` are
discrete time intervals between data points.

Basic usage
-----------

This example uses the files :data:`~MDAnalysis.tests.datafiles.PRM_NCBOX` and
:data:`~MDAnalysis.tests.datafiles.TRJ_NCBOX` from the MDAnalysis test suite.
To get started, execute  ::

   # imports
   >>> import MDAnalysis as mda
   >>> from transport_analysis.velocityautocorr import VelocityAutocorr

   # test data for this example
   >>> from MDAnalysis.tests.datafiles import PRM_NCBOX, TRJ_NCBOX

We will calculate the VACF of an atom group of all water atoms in
residues 1-5. To select these atoms:

   >>> u = mda.Universe(PRM_NCBOX, TRJ_NCBOX)
   >>> ag = u.select_atoms("resname WAT and resid 1-5")

We can run the calculation using any variable of choice such as
``wat_vacf`` and access our results using ``wat_vacf.results.timeseries``:

   >>> wat_vacf = VelocityAutocorr(ag).run()
   >>> wat_vacf.results.timeseries
   array([275.62075467, -18.42008255, -23.94383428,  41.41415381,
        -2.3164344 , -35.66393559, -22.66874897,  -3.97575003,
         6.57888933,  -5.29065096])

Notice that this example data is insufficient to provide a well-defined VACF.
When working with real data, ensure that the frames are captured frequently
enough to obtain a VACF suitable for your needs.

"""
from typing import TYPE_CHECKING

from MDAnalysis.analysis.base import AnalysisBase
from MDAnalysis.core.groups import UpdatingAtomGroup
from MDAnalysis.exceptions import NoDataError
import numpy as np
import tidynamics
import matplotlib.pyplot as plt
from scipy import integrate
from transport_analysis.due import due, Doi

if TYPE_CHECKING:
    from MDAnalysis.core.universe import AtomGroup

due.cite(
    Doi("10.21105/joss.00877"),
    description="Autocorrelation with tidynamics",
    path="transport_analysis.velocityautocorr",
    cite_module=True,
)


class VelocityAutocorr(AnalysisBase):
    """
    Class to calculate a velocity autocorrelation function (VACF).

    Parameters
    ----------
    atomgroup : AtomGroup
        An MDAnalysis :class:`~MDAnalysis.core.groups.AtomGroup`.
        Note that :class:`UpdatingAtomGroup` instances are not accepted.
    dim_type : {'xyz', 'xy', 'yz', 'xz', 'x', 'y', 'z'}
        Desired dimensions to be included in the VACF. Defaults to 'xyz'.
    fft : bool
        If ``True``, uses a fast FFT based algorithm for computation of
        the VACF. Otherwise, use the simple "windowed" algorithm.
        The tidynamics package is required for `fft=True`.
        Defaults to ``True``.

    Attributes
    ----------
    atomgroup : :class:`~MDAnalysis.core.groups.AtomGroup`
        The atoms to which this analysis is applied
    dim_fac : int
        Dimensionality :math:`d` of the VACF.
    results.timeseries : :class:`numpy.ndarray`
        The averaged VACF over all the particles with respect to lag-time.
        Obtained after calling :meth:`VelocityAutocorr.run`
    results.vacf_by_particle : :class:`numpy.ndarray`
        The VACF of each individual particle with respect to lag-time.
    start : Optional[int]
        The first frame of the trajectory used to compute the analysis.
    stop : Optional[int]
        The frame to stop at for the analysis, non-inclusive.
    step : Optional[int]
        Number of frames to skip between each analyzed frame.
    n_frames : int
        Number of frames analysed in the trajectory.
    n_particles : int
        Number of particles VACF was calculated over.
    """

    def __init__(
        self,
        atomgroup: "AtomGroup",
        dim_type: str = "xyz",
        fft: bool = True,
        **kwargs,
    ) -> None:
        # the below line must be kept to initialize the AnalysisBase class!
        super().__init__(atomgroup.universe.trajectory, **kwargs)
        # after this you will be able to access `self.results`
        # `self.results` is a dictionary-like object
        # that can should used to store and retrieve results
        # See more at the MDAnalysis documentation:
        # https://docs.mdanalysis.org/stable/documentation_pages/analysis/base.html?highlight=results#MDAnalysis.analysis.base.Results

        if isinstance(atomgroup, UpdatingAtomGroup):
            raise TypeError(
                "UpdatingAtomGroups are not valid for VACF computation"
            )

        # args
        self.dim_type = dim_type.lower()
        self._dim, self.dim_fac = self._parse_dim_type(self.dim_type)
        self.fft = fft

        # local
        self.atomgroup = atomgroup
        self.n_particles = len(self.atomgroup)
        self._run_called = False

    def _prepare(self):
        """Set up velocity and VACF arrays before the analysis loop begins"""
        # 2D array of frames x particles
        self.results.vacf_by_particle = np.zeros(
            (self.n_frames, self.n_particles)
        )

        # 3D array of frames x particles x dimensionality
        self._velocities = np.zeros(
            (self.n_frames, self.n_particles, self.dim_fac)
        )
        # self.results.timeseries not set here

    @staticmethod
    def _parse_dim_type(dim_str):
        """Sets up the desired dimensionality of the VACF."""
        keys = {
            "x": [0],
            "y": [1],
            "z": [2],
            "xy": [0, 1],
            "xz": [0, 2],
            "yz": [1, 2],
            "xyz": [0, 1, 2],
        }

        try:
            _dim = keys[dim_str]
        except KeyError:
            raise ValueError(
                "invalid dim_type: {} specified, please specify one of xyz, "
                "xy, xz, yz, x, y, z".format(dim_str)
            )

        return _dim, len(_dim)

    def _single_frame(self):
        """Constructs array of velocities for VACF calculation."""
        # This runs once for each frame of the trajectory

        # The trajectory positions update automatically
        # You can access the frame number using self._frame_index

        # trajectory must have velocity information
        if not self._ts.has_velocities:
            raise NoDataError(
                "VACF computation requires velocities in the trajectory"
            )

        # set shape of velocity array
        self._velocities[self._frame_index] = self.atomgroup.velocities[
            :, self._dim
        ]

    # Results will be in units of (angstroms / ps)^2
    def _conclude(self):
        """Calculate the final results of the analysis"""
        # This is an optional method that runs after
        # _single_frame loops over the trajectory.
        # It is useful for calculating the final results
        # of the analysis.
        if self.fft:
            self._conclude_fft()
        else:
            self._conclude_simple()

    def _conclude_fft(self):  # with FFT, np.float64 bit prescision required.
        r"""Calculates the VACF via the FCA fast correlation algorithm."""
        for n in range(self.n_particles):
            self.results.vacf_by_particle[:, n] = tidynamics.acf(
                self._velocities[:, n, :]
            )
        self.results.timeseries = self.results.vacf_by_particle.mean(axis=1)
        self._run_called = True

    def _conclude_simple(self):
        r"""Calculates the VACF via the simple "windowed" algorithm."""
        # total frames in trajectory, use N for readability
        N = self.n_frames

        # iterate through all possible lagtimes up to N
        for lag in range(N):
            # get product of velocities shifted by "lag" frames
            veloc = (
                self._velocities[: N - lag, :, :]
                * self._velocities[lag:, :, :]
            )

            # dot product of x(, y, z) velocities per particle
            sum_veloc = np.sum(veloc, axis=-1)

            # average over # frames
            # update VACF by particle array
            self.results.vacf_by_particle[lag, :] = np.mean(sum_veloc, axis=0)
        # average over # particles and update results array
        self.results.timeseries = self.results.vacf_by_particle.mean(axis=1)
        self._run_called = True

    def plot_vacf(
        self,
        start=0,
        stop=0,
        step=1,
        xlabel="Time (ps)",
        ylabel="Velocity Autocorrelation Function (Å^2 / ps^2)",
    ):
        """
        Returns a velocity autocorrelation function (VACF) plot via
        ``Matplotlib``. Analysis must be run prior to plotting.

        Parameters
        ----------
        start : Optional[int]
            The first frame of ``self.results.timeseries``
            used for the plot.
        stop : Optional[int]
            The frame of ``self.results.timeseries`` to stop at
            for the plot, non-inclusive.
        step : Optional[int]
            Number of frames to skip between each plotted frame.
        xlabel : Optional[str]
            The x-axis label text. Defaults to "Time (ps)".
        ylabel : Optional[str]
            The y-axis label text.
            Defaults to "Velocity Autocorrelation Function (Å^2 / ps^2)".

        Returns
        -------
        :class:`matplotlib.lines.Line2D`
            A :class:`matplotlib.lines.Line2D` instance with
            the desired VACF plotting information.
        """
        if not self._run_called:
            raise RuntimeError("Analysis must be run prior to plotting")

        stop = self.n_frames if stop == 0 else stop

        fig, ax_vacf = plt.subplots()
        ax_vacf.set_xlabel(xlabel)
        ax_vacf.set_ylabel(ylabel)
        return ax_vacf.plot(
            self.times[start:stop:step],
            self.results.timeseries[start:stop:step],
        )

    def self_diffusivity_gk(self, start=0, stop=0, step=1):
        """
        Returns a self-diffusivity value using ``scipy.integrate.trapezoid``.
        Analysis must be run prior to computing self-diffusivity.

        Parameters
        ----------
        start : Optional[int]
            The first frame of ``self.results.timeseries``
            used for the calculation.
        stop : Optional[int]
            The frame of ``self.results.timeseries`` to stop at
            for the calculation, non-inclusive.
        step : Optional[int]
            Number of frames to skip between each frame used
            for the calculation.

        Returns
        -------
        `numpy.float64`
            The calculated self-diffusivity value for the analysis.
        """
        if not self._run_called:
            raise RuntimeError(
                "Analysis must be run prior to computing self-diffusivity"
            )

        stop = self.n_frames if stop == 0 else stop

        return (
            integrate.trapezoid(
                self.results.timeseries[start:stop:step],
                self.times[start:stop:step],
            )
            / self.dim_fac
        )

    def self_diffusivity_gk_odd(self, start=0, stop=0, step=1):
        """
        Returns a self-diffusivity value using ``scipy.integrate.simpson``.
        Recommended for use with an odd number of evenly spaced data points.
        Analysis must be run prior to computing self-diffusivity.

        Parameters
        ----------
        start : Optional[int]
            The first frame of ``self.results.timeseries``
            used for the calculation.
        stop : Optional[int]
            The frame of ``self.results.timeseries`` to stop at
            for the calculation, non-inclusive.
        step : Optional[int]
            Number of frames to skip between each frame used
            for the calculation.

        Returns
        -------
        `numpy.float64`
            The calculated self-diffusivity value for the analysis.
        """
        if not self._run_called:
            raise RuntimeError(
                "Analysis must be run prior to computing self-diffusivity"
            )

        stop = self.n_frames if stop == 0 else stop

        return (
            integrate.simpson(
                self.results.timeseries[start:stop:step],
                self.times[start:stop:step],
            )
            / self.dim_fac
        )

    def plot_running_integral(
        self,
        start=0,
        stop=0,
        step=1,
        initial=0,
        xlabel="Time (ps)",
        ylabel="Running Integral of the VACF (Å^2 / ps)",
    ):
        """
        Returns a plot of the running integral of the
        velocity autocorrelation function (VACF) via ``Matplotlib``.
        In this case, the running integral is the integral of the VACF
        divided by the dimensionality. Analysis must be run prior to plotting.

        Parameters
        ----------
        start : Optional[int]
            The first frame of ``self.results.timeseries``
            used for the plot.
        stop : Optional[int]
            The frame of ``self.results.timeseries`` to stop at
            for the plot, non-inclusive.
        step : Optional[int]
            Number of frames to skip between each plotted frame.
        initial : Optional[float]
            Inserted value at the beginning of the integrated result array.
            Defaults to 0.
        xlabel : Optional[str]
            The x-axis label text. Defaults to "Time (ps)".
        ylabel : Optional[str]
            The y-axis label text.
            Defaults to "Running Integral of the VACF (Å^2 / ps)".

        Returns
        -------
        :class:`matplotlib.lines.Line2D`
            A :class:`matplotlib.lines.Line2D` instance with
            the desired VACF plotting information.
        """
        if not self._run_called:
            raise RuntimeError("Analysis must be run prior to plotting")

        stop = self.n_frames if stop == 0 else stop

        running_integral = (
            integrate.cumulative_trapezoid(
                self.results.timeseries[start:stop:step],
                self.times[start:stop:step],
                initial=initial,
            )
            / self.dim_fac
        )

        fig, ax_running_integral = plt.subplots()
        ax_running_integral.set_xlabel(xlabel)
        ax_running_integral.set_ylabel(ylabel)
        return ax_running_integral.plot(
            self.times[start:stop:step],
            running_integral,
        )



"""
Transport Analysis
A Python package to compute and analyze transport properties.
"""
import os
import sys
from setuptools import setup, find_packages

sys.path.append(os.path.dirname(__file__))

import versioneer

short_description = "A Python package to compute and analyze transport properties.".strip().split("\n")[0]

# from https://github.com/pytest-dev/pytest-runner#conditional-requirement
needs_pytest = {'pytest', 'test', 'ptr'}.intersection(sys.argv)
pytest_runner = ['pytest-runner'] if needs_pytest else []

try:
    with open("README.md", "r") as handle:
        long_description = handle.read()
except:
    long_description = "\n".join(short_description[2:])


setup(
    # Self-descriptive entries which should always be present
    name='transport_analysis',
    author='Xu Hong Chen',
    author_email='xhgchen@gmail.com',
    description=short_description,
    long_description=long_description,
    long_description_content_type="text/markdown",
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    license='GNU Public License v2+',

    # Which Python importable modules should be included when your package is installed
    # Handled automatically by setuptools. Use 'exclude' to prevent some specific
    # subpackage(s) from being added, if needed
    packages=find_packages(),

    # Optional include package data to ship with your package
    # Customize MANIFEST.in if the general case does not suit your needs
    # Comment out this line to prevent the files from being packaged with your software
    include_package_data=True,
    python_requires=">=3.9",          # Python version restrictions
    # Allows `setup.py test` to work correctly with pytest
    setup_requires=[] + pytest_runner,
    # Required packages, pulls from pip if needed
    # do not use for Conda deployment
    install_requires=[
        "mdanalysis>=2.0.0",
        "tidynamics>=1.0.0",
    ],
    # Additional entries you may want simply uncomment the lines you want and fill in the data
    # url='transport_analysis.readthedocs.io/en/latest/',  # Website
    # platforms=['Linux',
    #            'Mac OS-X',
    #            'Unix',
    #            'Windows'],            # Valid platforms your code works on, adjust to your flavor

    # Manual control if final package is compressible or not, set False to prevent the .egg from being made
    # zip_safe=False,

    extras_require={
        "test": [
            "pytest>=6.0",
            "pytest-xdist>=2.5",
            "pytest-cov>=3.0",
            "MDAnalysisTests>=2.0.0",
        ],
        "doc": [
            "sphinx",
            "sphinx_rtd_theme",
        ]
    }
)
